@@ -1,10 +1,17 @@
 {-# LANGUAGE CPP #-}
 {-# LANGUAGE FlexibleContexts #-}
 {-# LANGUAGE FlexibleInstances #-}
+{-# LANGUAGE KindSignatures #-}
 {-# LANGUAGE TypeOperators #-}
 {-# LANGUAGE TypeSynonymInstances #-}
+
 #if __GLASGOW_HASKELL__ >= 701
 {-# LANGUAGE DefaultSignatures #-}
+{-# LANGUAGE Trustworthy #-}
+#endif
+
+#if __GLASGOW_HASKELL__ >= 705
+{-# LANGUAGE PolyKinds #-}
 #endif
 
 #include "HsBaseConfig.h"
@@ -25,29 +32,43 @@ module Generics.Deriving.Enum (
 
   ) where
 
-import Control.Applicative (Const, ZipList)
+import           Control.Applicative (Const, ZipList)
 
-import Data.Int
-import Data.Monoid (All, Any, Dual, First, Last, Product, Sum)
+import           Data.Int
+import           Data.Monoid (All, Any, Dual, Product, Sum)
+import qualified Data.Monoid as Monoid (First, Last)
 import Data.Word
 
-import Foreign.C.Types
-import Foreign.Ptr
+import           Foreign.C.Types
+import           Foreign.Ptr
 
-import Generics.Deriving.Base
-import Generics.Deriving.Instances ()
-import Generics.Deriving.Eq
+import           Generics.Deriving.Base
+import           Generics.Deriving.Eq
 
-import System.Posix.Types
+import           System.Exit (ExitCode)
+import           System.Posix.Types
+
+#if MIN_VERSION_base(4,4,0)
+import           Data.Complex (Complex)
+#endif
 
 #if MIN_VERSION_base(4,7,0)
-import Data.Proxy (Proxy)
+import           Data.Coerce (coerce)
+import           Data.Proxy (Proxy)
+#else
+import           Unsafe.Coerce (unsafeCoerce)
 #endif
 
 #if MIN_VERSION_base(4,8,0)
-import Data.Functor.Identity (Identity)
-import Data.Monoid (Alt)
-import Numeric.Natural (Natural)
+import           Data.Functor.Identity (Identity)
+import           Data.Monoid (Alt)
+import           Numeric.Natural (Natural)
+#endif
+
+#if MIN_VERSION_base(4,9,0)
+import           Data.List.NonEmpty (NonEmpty)
+import qualified Data.Semigroup as Semigroup (First, Last)
+import           Data.Semigroup (Arg, Max, Min, Option, WrappedMonoid)
 #endif
 
 -----------------------------------------------------------------------------
@@ -132,9 +153,21 @@ class GEnum a where
   genum = genumDefault
 #endif
 
-genumNum :: (Enum a, Num a) => [a]
-genumNum = [0..] ||| (neg 0) where
-  neg n = (n-1) : neg (n-1)
+genumNumUnbounded :: Num a => [a]
+genumNumUnbounded = pos 0 ||| neg 0 where
+  pos n = n     : pos (n + 1)
+  neg n = (n-1) : neg (n - 1)
+
+genumNumSigned :: (Bounded a, Enum a, Num a) => [a]
+genumNumSigned = [0 .. maxBound] ||| [-1, -2 .. minBound]
+
+genumNumUnsigned :: (Enum a, Num a) => [a]
+genumNumUnsigned = [0 ..]
+
+#if !(MIN_VERSION_base(4,7,0))
+coerce :: a -> b
+coerce = unsafeCoerce
+#endif
 
 -- Base types instances
 instance GEnum () where
@@ -183,8 +216,15 @@ instance GEnum (f a) => GEnum (Alt f a) where
 instance GEnum Any where
   genum = genumDefault
 
+#if MIN_VERSION_base(4,9,0)
+instance (GEnum a, GEnum b) => GEnum (Arg a b) where
+  genum = genumDefault
+#endif
+
+#if !(MIN_VERSION_base(4,9,0))
 instance GEnum Arity where
   genum = genumDefault
+#endif
 
 instance GEnum Associativity where
   genum = genumDefault
@@ -194,64 +234,69 @@ instance GEnum Bool where
 
 #if defined(HTYPE_CC_T)
 instance GEnum CCc where
-  genum = genumNum
+  genum = coerce (genum :: [HTYPE_CC_T])
 #endif
 
 instance GEnum CChar where
-  genum = genumNum
+  genum = coerce (genum :: [HTYPE_CHAR])
 
 instance GEnum CClock where
-  genum = genumNum
+  genum = coerce (genum :: [HTYPE_CLOCK_T])
 
 #if defined(HTYPE_DEV_T)
 instance GEnum CDev where
-  genum = genumNum
+  genum = coerce (genum :: [HTYPE_DEV_T])
 #endif
 
 instance GEnum CDouble where
-  genum = genumNum
+  genum = coerce (genum :: [HTYPE_DOUBLE])
 
 instance GEnum CFloat where
-  genum = genumNum
+  genum = coerce (genum :: [HTYPE_FLOAT])
 
 #if defined(HTYPE_GID_T)
 instance GEnum CGid where
-  genum = genumNum
+  genum = coerce (genum :: [HTYPE_GID_T])
 #endif
 
 #if defined(HTYPE_INO_T)
 instance GEnum CIno where
-  genum = genumNum
+  genum = coerce (genum :: [HTYPE_INO_T])
 #endif
 
 instance GEnum CInt where
-  genum = genumNum
+  genum = coerce (genum :: [HTYPE_INT])
 
 instance GEnum CIntMax where
-  genum = genumNum
+  genum = coerce (genum :: [HTYPE_INTMAX_T])
 
 instance GEnum CIntPtr where
-  genum = genumNum
+  genum = coerce (genum :: [HTYPE_INTPTR_T])
 
 instance GEnum CLLong where
-  genum = genumNum
+  genum = coerce (genum :: [HTYPE_LONG_LONG])
 
 instance GEnum CLong where
-  genum = genumNum
+  genum = coerce (genum :: [HTYPE_LONG])
 
 #if defined(HTYPE_MODE_T)
 instance GEnum CMode where
-  genum = genumNum
+  genum = coerce (genum :: [HTYPE_MODE_T])
 #endif
 
 #if defined(HTYPE_NLINK_T)
 instance GEnum CNlink where
-  genum = genumNum
+  genum = coerce (genum :: [HTYPE_NLINK_T])
 #endif
 
 #if defined(HTYPE_OFF_T)
 instance GEnum COff where
-  genum = genumNum
+  genum = coerce (genum :: [HTYPE_OFF_T])
+#endif
+
+#if MIN_VERSION_base(4,4,0)
+instance GEnum a => GEnum (Complex a) where
+  genum = genumDefault
 #endif
 
 instance GEnum a => GEnum (Const a b) where
@@ -259,88 +304,88 @@ instance GEnum a => GEnum (Const a b) where
 
 #if defined(HTYPE_PID_T)
 instance GEnum CPid where
-  genum = genumNum
+  genum = coerce (genum :: [HTYPE_PID_T])
 #endif
 
 instance GEnum CPtrdiff where
-  genum = genumNum
+  genum = coerce (genum :: [HTYPE_PTRDIFF_T])
 
 #if defined(HTYPE_RLIM_T)
 instance GEnum CRLim where
-  genum = genumNum
+  genum = coerce (genum :: [HTYPE_RLIM_T])
 #endif
 
 instance GEnum CSChar where
-  genum = genumNum
+  genum = coerce (genum :: [HTYPE_SIGNED_CHAR])
 
 #if defined(HTYPE_SPEED_T)
 instance GEnum CSpeed where
-  genum = genumNum
+  genum = coerce (genum :: [HTYPE_SPEED_T])
 #endif
 
 #if MIN_VERSION_base(4,4,0)
 instance GEnum CSUSeconds where
-  genum = genumNum
+  genum = coerce (genum :: [HTYPE_SUSECONDS_T])
 #endif
 
 instance GEnum CShort where
-  genum = genumNum
+  genum = coerce (genum :: [HTYPE_SHORT])
 
 instance GEnum CSigAtomic where
-  genum = genumNum
+  genum = coerce (genum :: [HTYPE_SIG_ATOMIC_T])
 
 instance GEnum CSize where
-  genum = genumNum
+  genum = coerce (genum :: [HTYPE_SIZE_T])
 
 #if defined(HTYPE_SSIZE_T)
 instance GEnum CSsize where
-  genum = genumNum
+  genum = coerce (genum :: [HTYPE_SSIZE_T])
 #endif
 
 #if defined(HTYPE_TCFLAG_T)
 instance GEnum CTcflag where
-  genum = genumNum
+  genum = coerce (genum :: [HTYPE_TCFLAG_T])
 #endif
 
 instance GEnum CTime where
-  genum = genumNum
+  genum = coerce (genum :: [HTYPE_TIME_T])
 
 instance GEnum CUChar where
-  genum = genumNum
+  genum = coerce (genum :: [HTYPE_UNSIGNED_CHAR])
 
 #if defined(HTYPE_UID_T)
 instance GEnum CUid where
-  genum = genumNum
+  genum = coerce (genum :: [HTYPE_UID_T])
 #endif
 
 instance GEnum CUInt where
-  genum = genumNum
+  genum = coerce (genum :: [HTYPE_UNSIGNED_INT])
 
 instance GEnum CUIntMax where
-  genum = genumNum
+  genum = coerce (genum :: [HTYPE_UINTMAX_T])
 
 instance GEnum CUIntPtr where
-  genum = genumNum
+  genum = coerce (genum :: [HTYPE_UINTPTR_T])
 
 instance GEnum CULLong where
-  genum = genumNum
+  genum = coerce (genum :: [HTYPE_UNSIGNED_LONG_LONG])
 
 instance GEnum CULong where
-  genum = genumNum
+  genum = coerce (genum :: [HTYPE_UNSIGNED_LONG])
 
 #if MIN_VERSION_base(4,4,0)
 instance GEnum CUSeconds where
-  genum = genumNum
+  genum = coerce (genum :: [HTYPE_USECONDS_T])
 #endif
 
 instance GEnum CUShort where
-  genum = genumNum
+  genum = coerce (genum :: [HTYPE_UNSIGNED_SHORT])
 
 instance GEnum CWchar where
-  genum = genumNum
+  genum = coerce (genum :: [HTYPE_WCHAR_T])
 
 instance GEnum Double where
-  genum = genumNum
+  genum = genumNumUnbounded
 
 instance GEnum a => GEnum (Dual a) where
   genum = genumDefault
@@ -348,17 +393,25 @@ instance GEnum a => GEnum (Dual a) where
 instance (GEnum a, GEnum b) => GEnum (Either a b) where
   genum = genumDefault
 
-instance GEnum Fd where
-  genum = genumNum
-
-instance GEnum a => GEnum (First a) where
+instance GEnum ExitCode where
   genum = genumDefault
 
+instance GEnum Fd where
+  genum = coerce (genum :: [CInt])
+
+instance GEnum a => GEnum (Monoid.First a) where
+  genum = genumDefault
+
+#if MIN_VERSION_base(4,9,0)
+instance GEnum a => GEnum (Semigroup.First a) where
+  genum = genumDefault
+#endif
+
 instance GEnum Fixity where
   genum = genumDefault
 
 instance GEnum Float where
-  genum = genumNum
+  genum = genumNumUnbounded
 
 #if MIN_VERSION_base(4,8,0)
 instance GEnum a => GEnum (Identity a) where
@@ -366,38 +419,66 @@ instance GEnum a => GEnum (Identity a) where
 #endif
 
 instance GEnum Int where
-  genum = genumNum
+  genum = genumNumSigned
 
 instance GEnum Int8 where
-  genum = genumNum
+  genum = genumNumSigned
 
 instance GEnum Int16 where
-  genum = genumNum
+  genum = genumNumSigned
 
 instance GEnum Int32 where
-  genum = genumNum
+  genum = genumNumSigned
 
 instance GEnum Int64 where
-  genum = genumNum
+  genum = genumNumSigned
 
 instance GEnum Integer where
-  genum = genumNum
+  genum = genumNumUnbounded
 
 instance GEnum IntPtr where
-  genum = genumNum
+  genum = genumNumSigned
 
 instance GEnum c => GEnum (K1 i c p) where
   genum = genumDefault
 
-instance GEnum a => GEnum (Last a) where
+instance GEnum a => GEnum (Monoid.Last a) where
   genum = genumDefault
 
+#if MIN_VERSION_base(4,9,0)
+instance GEnum a => GEnum (Semigroup.Last a) where
+  genum = genumDefault
+#endif
+
 instance GEnum (f p) => GEnum (M1 i c f p) where
   genum = genumDefault
 
+#if MIN_VERSION_base(4,9,0)
+instance GEnum a => GEnum (Max a) where
+  genum = genumDefault
+#endif
+
 instance GEnum a => GEnum (Maybe a) where
   genum = genumDefault
 
+#if MIN_VERSION_base(4,9,0)
+instance GEnum a => GEnum (Min a) where
+  genum = genumDefault
+#endif
+
+#if MIN_VERSION_base(4,8,0)
+instance GEnum Natural where
+  genum = genumNumUnsigned
+#endif
+
+#if MIN_VERSION_base(4,9,0)
+instance GEnum a => GEnum (NonEmpty a) where
+  genum = genumDefault
+
+instance GEnum a => GEnum (Option a) where
+  genum = genumDefault
+#endif
+
 instance GEnum Ordering where
   genum = genumDefault
 
@@ -408,7 +489,13 @@ instance GEnum a => GEnum (Product a) where
   genum = genumDefault
 
 #if MIN_VERSION_base(4,7,0)
-instance GEnum (Proxy s) where
+instance GEnum
+# if MIN_VERSION_base(4,9,0)
+               (Proxy s)
+# else
+               (Proxy (s :: *))
+# endif
+               where
   genum = genumDefault
 #endif
 
@@ -422,22 +509,27 @@ instance GEnum (U1 p) where
   genum = genumDefault
 
 instance GEnum Word where
-  genum = genumNum
+  genum = genumNumUnsigned
 
 instance GEnum Word8 where
-  genum = genumNum
+  genum = genumNumUnsigned
 
 instance GEnum Word16 where
-  genum = genumNum
+  genum = genumNumUnsigned
 
 instance GEnum Word32 where
-  genum = genumNum
+  genum = genumNumUnsigned
 
 instance GEnum Word64 where
-  genum = genumNum
+  genum = genumNumUnsigned
 
 instance GEnum WordPtr where
-  genum = genumNum
+  genum = genumNumUnsigned
+
+#if MIN_VERSION_base(4,9,0)
+instance GEnum m => GEnum (WrappedMonoid m) where
+  genum = genumDefault
+#endif
 
 instance GEnum a => GEnum (ZipList a) where
   genum = genumDefault
@@ -573,10 +665,19 @@ instance GIx Any where
   index   = indexDefault
   inRange = inRangeDefault
 
+#if MIN_VERSION_base(4,9,0)
+instance (GEq a, GEnum a, GIx a, GEnum b) => GIx (Arg a b) where
+  range   = rangeDefault
+  index   = indexDefault
+  inRange = inRangeDefault
+#endif
+
+#if !(MIN_VERSION_base(4,9,0))
 instance GIx Arity where
   range   = rangeDefault
   index   = indexDefault
   inRange = inRangeDefault
+#endif
 
 instance GIx Associativity where
   range   = rangeDefault
@@ -763,16 +864,28 @@ instance (GEq a, GEnum a, GIx a, GEq b, GEnum b, GIx b) => GIx (Either a b) wher
   index   = indexDefault
   inRange = inRangeDefault
 
+instance GIx ExitCode where
+  range   = rangeDefault
+  index   = indexDefault
+  inRange = inRangeDefault
+
 instance GIx Fd where
   range   = rangeEnum
   index   = indexIntegral
   inRange = inRangeOrd
 
-instance (GEq a, GEnum a, GIx a) => GIx (First a) where
+instance (GEq a, GEnum a, GIx a) => GIx (Monoid.First a) where
   range   = rangeDefault
   index   = indexDefault
   inRange = inRangeDefault
 
+#if MIN_VERSION_base(4,9,0)
+instance (GEq a, GEnum a, GIx a) => GIx (Semigroup.First a) where
+  range   = rangeDefault
+  index   = indexDefault
+  inRange = inRangeDefault
+#endif
+
 instance GIx Fixity where
   range   = rangeDefault
   index   = indexDefault
@@ -820,16 +933,37 @@ instance GIx IntPtr where
   index   = indexIntegral
   inRange = inRangeOrd
 
-instance (GEq a, GEnum a, GIx a) => GIx (Last a) where
+instance (GEq a, GEnum a, GIx a) => GIx (Monoid.Last a) where
   range   = rangeDefault
   index   = indexDefault
   inRange = inRangeDefault
 
+#if MIN_VERSION_base(4,9,0)
+instance (GEq a, GEnum a, GIx a) => GIx (Semigroup.Last a) where
+  range   = rangeDefault
+  index   = indexDefault
+  inRange = inRangeDefault
+#endif
+
+#if MIN_VERSION_base(4,9,0)
+instance (GEq a, GEnum a, GIx a) => GIx (Max a) where
+  range   = rangeDefault
+  index   = indexDefault
+  inRange = inRangeDefault
+#endif
+
 instance (GEq a, GEnum a, GIx a) => GIx (Maybe a) where
   range   = rangeDefault
   index   = indexDefault
   inRange = inRangeDefault
 
+#if MIN_VERSION_base(4,9,0)
+instance (GEq a, GEnum a, GIx a) => GIx (Min a) where
+  range   = rangeDefault
+  index   = indexDefault
+  inRange = inRangeDefault
+#endif
+
 #if MIN_VERSION_base(4,8,0)
 instance GIx Natural where
   range   = rangeEnum
@@ -837,6 +971,20 @@ instance GIx Natural where
   inRange = inRangeOrd
 #endif
 
+#if MIN_VERSION_base(4,9,0)
+instance (GEq a, GEnum a, GIx a) => GIx (NonEmpty a) where
+  range   = rangeDefault
+  index   = indexDefault
+  inRange = inRangeDefault
+#endif
+
+#if MIN_VERSION_base(4,9,0)
+instance (GEq a, GEnum a, GIx a) => GIx (Option a) where
+  range   = rangeDefault
+  index   = indexDefault
+  inRange = inRangeDefault
+#endif
+
 instance GIx Ordering where
   range   = rangeDefault
   index   = indexDefault
@@ -848,7 +996,13 @@ instance (GEq a, GEnum a, GIx a) => GIx (Product a) where
   inRange = inRangeDefault
 
 #if MIN_VERSION_base(4,7,0)
-instance GIx (Proxy s) where
+instance GIx
+# if MIN_VERSION_base(4,9,0)
+             (Proxy s)
+# else
+             (Proxy (s :: *))
+# endif
+             where
   range   = rangeDefault
   index   = indexDefault
   inRange = inRangeDefault
@@ -888,3 +1042,10 @@ instance GIx WordPtr where
   range   = rangeEnum
   index   = indexIntegral
   inRange = inRangeOrd
+
+#if MIN_VERSION_base(4,9,0)
+instance (GEq m, GEnum m, GIx m) => GIx (WrappedMonoid m) where
+  range   = rangeDefault
+  index   = indexDefault
+  inRange = inRangeDefault
+#endif
