@@ -2,11 +2,13 @@
 {-# LANGUAGE FlexibleContexts #-}
 {-# LANGUAGE FlexibleInstances #-}
 {-# LANGUAGE MagicHash #-}
+{-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE TypeOperators #-}
 {-# LANGUAGE TypeSynonymInstances #-}
 
 #if __GLASGOW_HASKELL__ >= 701
 {-# LANGUAGE DefaultSignatures #-}
+{-# LANGUAGE Trustworthy #-}
 #endif
 
 #if __GLASGOW_HASKELL__ < 709
@@ -24,34 +26,45 @@ module Generics.Deriving.Show (
 
 import Control.Applicative (Const, ZipList)
 
-import Data.Char (GeneralCategory)
-import Data.Int
-import Data.Monoid (All, Any, Dual, First, Last, Product, Sum)
-import Data.Word
+import           Data.Char (GeneralCategory)
+import           Data.Int
+import           Data.Monoid (All, Any, Dual, Product, Sum)
+import qualified Data.Monoid as Monoid (First, Last)
+import           Data.Version (Version)
+import           Data.Word
 
-import Foreign.C.Types
-import Foreign.ForeignPtr (ForeignPtr)
-import Foreign.Ptr
+import           Foreign.C.Types
+import           Foreign.ForeignPtr (ForeignPtr)
+import           Foreign.Ptr
 
-import Generics.Deriving.Base
-import Generics.Deriving.Instances ()
+import           Generics.Deriving.Base
 
-import GHC.Exts hiding (Any)
+import           GHC.Exts hiding (Any)
 
-import System.Exit (ExitCode)
-import System.IO (BufferMode, Handle, HandlePosn, IOMode, SeekMode)
-import System.IO.Error (IOErrorType)
-import System.Posix.Types
+import           System.Exit (ExitCode)
+import           System.IO (BufferMode, Handle, HandlePosn, IOMode, SeekMode)
+import           System.IO.Error (IOErrorType)
+import           System.Posix.Types
+
+#if MIN_VERSION_base(4,4,0)
+import           Data.Complex (Complex)
+#endif
 
 #if MIN_VERSION_base(4,7,0)
-import Data.Proxy (Proxy)
+import           Data.Proxy (Proxy)
 #endif
 
 #if MIN_VERSION_base(4,8,0)
-import Data.Functor.Identity (Identity)
-import Data.Monoid (Alt)
-import Data.Void (Void)
-import Numeric.Natural (Natural)
+import           Data.Functor.Identity (Identity)
+import           Data.Monoid (Alt)
+import           Data.Void (Void)
+import           Numeric.Natural (Natural)
+#endif
+
+#if MIN_VERSION_base(4,9,0)
+import           Data.List.NonEmpty (NonEmpty)
+import qualified Data.Semigroup as Semigroup (First, Last)
+import           Data.Semigroup (Arg, Max, Min, Option, WrappedMonoid)
 #endif
 
 --------------------------------------------------------------------------------
@@ -103,6 +116,8 @@ instance (GShow' a, Constructor c) => GShow' (M1 C c a) where
             showBraces Tup     p = showChar '(' . p . showChar ')'
             showBraces Pref    p = p
             showBraces (Inf _) p = p
+
+            conIsTuple :: C1 c f p -> Bool
             conIsTuple y = tupleName (conName y) where
               tupleName ('(':',':_) = True
               tupleName _           = False
@@ -220,8 +235,15 @@ instance GShow (f a) => GShow (Alt f a) where
 instance GShow Any where
   gshowsPrec = gshowsPrecdefault
 
+#if MIN_VERSION_base(4,9,0)
+instance (GShow a, GShow b) => GShow (Arg a b) where
+  gshowsPrec = gshowsPrecdefault
+#endif
+
+#if !(MIN_VERSION_base(4,9,0))
 instance GShow Arity where
   gshowsPrec = gshowsPrecdefault
+#endif
 
 instance GShow Associativity where
   gshowsPrec = gshowsPrecdefault
@@ -297,6 +319,11 @@ instance GShow COff where
   gshowsPrec = showsPrec
 #endif
 
+#if MIN_VERSION_base(4,4,0)
+instance GShow a => GShow (Complex a) where
+  gshowsPrec = gshowsPrecdefault
+#endif
+
 instance GShow a => GShow (Const a b) where
   gshowsPrec = gshowsPrecdefault
 
@@ -392,14 +419,19 @@ instance (GShow a, GShow b) => GShow (Either a b) where
   gshowsPrec = gshowsPrecdefault
 
 instance GShow ExitCode where
-  gshowsPrec = showsPrec
+  gshowsPrec = gshowsPrecdefault
 
 instance GShow Fd where
   gshowsPrec = showsPrec
 
-instance GShow a => GShow (First a) where
+instance GShow a => GShow (Monoid.First a) where
   gshowsPrec = gshowsPrecdefault
 
+#if MIN_VERSION_base(4,9,0)
+instance GShow a => GShow (Semigroup.First a) where
+  gshowsPrec = gshowsPrecdefault
+#endif
+
 instance GShow Fixity where
   gshowsPrec = gshowsPrecdefault
 
@@ -459,20 +491,43 @@ instance GShow IOMode where
 instance GShow c => GShow (K1 i c p) where
   gshowsPrec = gshowsPrecdefault
 
-instance GShow a => GShow (Last a) where
+instance GShow a => GShow (Monoid.Last a) where
   gshowsPrec = gshowsPrecdefault
 
+#if MIN_VERSION_base(4,9,0)
+instance GShow a => GShow (Semigroup.Last a) where
+  gshowsPrec = gshowsPrecdefault
+#endif
+
 instance GShow (f p) => GShow (M1 i c f p) where
   gshowsPrec = gshowsPrecdefault
 
+#if MIN_VERSION_base(4,9,0)
+instance GShow a => GShow (Max a) where
+  gshowsPrec = gshowsPrecdefault
+#endif
+
 instance GShow a => GShow (Maybe a) where
   gshowsPrec = gshowsPrecdefault
 
+#if MIN_VERSION_base(4,9,0)
+instance GShow a => GShow (Min a) where
+  gshowsPrec = gshowsPrecdefault
+#endif
+
 #if MIN_VERSION_base(4,8,0)
 instance GShow Natural where
   gshowsPrec = showsPrec
 #endif
 
+#if MIN_VERSION_base(4,9,0)
+instance GShow a => GShow (NonEmpty a) where
+  gshowsPrec = gshowsPrecdefault
+
+instance GShow a => GShow (Option a) where
+  gshowsPrec = gshowsPrecdefault
+#endif
+
 instance GShow Ordering where
   gshowsPrec = gshowsPrecdefault
 
@@ -509,6 +564,24 @@ instance GShow a => GShow (Sum a) where
 instance GShow (U1 p) where
   gshowsPrec = gshowsPrecdefault
 
+instance GShow (UChar p) where
+  gshowsPrec = gshowsPrecdefault
+
+instance GShow (UDouble p) where
+  gshowsPrec = gshowsPrecdefault
+
+instance GShow (UFloat p) where
+  gshowsPrec = gshowsPrecdefault
+
+instance GShow (UInt p) where
+  gshowsPrec = gshowsPrecdefault
+
+instance GShow (UWord p) where
+  gshowsPrec = gshowsPrecdefault
+
+instance GShow Version where
+  gshowsPrec = gshowsPrecdefault
+
 #if MIN_VERSION_base(4,8,0)
 instance GShow Void where
   gshowsPrec = showsPrec
@@ -532,5 +605,10 @@ instance GShow Word64 where
 instance GShow WordPtr where
   gshowsPrec = showsPrec
 
+#if MIN_VERSION_base(4,9,0)
+instance GShow m => GShow (WrappedMonoid m) where
+  gshowsPrec = gshowsPrecdefault
+#endif
+
 instance GShow a => GShow (ZipList a) where
   gshowsPrec = gshowsPrecdefault
